<?php
/**
* @package RSForm!Pro
* @copyright (C) 2007-2019 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

// no direct access
defined('_JEXEC') or die('Restricted access');

define('RSFORM_FIELD_RSEVENTS_NAME', 30);
define('RSFORM_FIELD_RSEVENTS_EMAIL', 31);
define('RSFORM_FIELD_RSEVENTS_TICKETS', 32);
define('RSFORM_FIELD_RSEVENTS_PAYMENTS', 33);
define('RSFORM_FIELD_RSEVENTS_COUPON', 34);

class plgSystemRsfprseventspro extends JPlugin
{
	protected $autoloadLanguage = true;

	public $newComponents = array(
		RSFORM_FIELD_RSEVENTS_NAME,
		RSFORM_FIELD_RSEVENTS_EMAIL,
		RSFORM_FIELD_RSEVENTS_TICKETS,
		RSFORM_FIELD_RSEVENTS_PAYMENTS,
		RSFORM_FIELD_RSEVENTS_COUPON
	);

	public function onRsformBackendAfterCreateFieldGroups(&$fieldGroups, $self)
	{
		$formId = JFactory::getApplication()->input->getInt('formId');

		$fieldGroups['rseventspro'] = (object) array(
			'name' => JText::_('RSFP_RSEPRO_LABEL'),
			'fields' => array()
		);

		$exists = RSFormProHelper::componentExists($formId, RSFORM_FIELD_RSEVENTS_NAME);
		$fieldGroups['rseventspro']->fields[] = (object) array(
			'id' 	=> RSFORM_FIELD_RSEVENTS_NAME,
			'name' 	=> JText::_('RSFP_RSEPRO_NAME'),
			'icon'  => 'rsficon rsficon-progress-full',
			'exists' => $exists ? $exists[0] : false
		);
		$exists = RSFormProHelper::componentExists($formId, RSFORM_FIELD_RSEVENTS_EMAIL);
		$fieldGroups['rseventspro']->fields[] = (object) array(
			'id' 	=> RSFORM_FIELD_RSEVENTS_EMAIL,
			'name' 	=> JText::_('RSFP_RSEPRO_EMAIL'),
			'icon'  => 'rsficon rsficon-envelope-square',
			'exists' => $exists ? $exists[0] : false
		);
		$exists = RSFormProHelper::componentExists($formId, RSFORM_FIELD_RSEVENTS_TICKETS);
		$fieldGroups['rseventspro']->fields[] = (object) array(
			'id' 	=> RSFORM_FIELD_RSEVENTS_TICKETS,
			'name' 	=> JText::_('RSFP_RSEPRO_TICKETS'),
			'icon'  => 'rsficon rsficon-ticket',
			'exists' => $exists ? $exists[0] : false
		);
		$exists = RSFormProHelper::componentExists($formId, RSFORM_FIELD_RSEVENTS_PAYMENTS);
		$fieldGroups['rseventspro']->fields[] = (object) array(
			'id' 	=> RSFORM_FIELD_RSEVENTS_PAYMENTS,
			'name' 	=> JText::_('RSFP_RSEPRO_PAYMENTS'),
			'icon'  => 'rsficon rsficon-caret-square-o-down',
			'exists' => $exists ? $exists[0] : false
		);
		$exists = RSFormProHelper::componentExists($formId, RSFORM_FIELD_RSEVENTS_COUPON);
		$fieldGroups['rseventspro']->fields[] = (object) array(
			'id' 	=> RSFORM_FIELD_RSEVENTS_COUPON,
			'name' 	=> JText::_('RSFP_RSEPRO_COUPON'),
			'icon'  => 'rsficon rsficon-crop_7_5',
			'exists' => $exists ? $exists[0] : false
		);
	}

	// AJAX Form Validation
	public function onRsformFrontendAJAXScriptCreate($args)
	{
		if (JFactory::getApplication()->input->get('option') === 'com_rseventspro' && $this->hasEventFields($args['formId']))
		{
			$args['script'] .= 'if (typeof ajaxValidationRSEventsPro !== \'undefined\') ajaxValidationRSEventsPro(task, formId, data);';
		}
	}

	// After store submissions
	public function onRsformFrontendAfterStoreSubmissions($args)
	{
		$jinput	= JFactory::getApplication()->input;

		if (!self::canRun() || $jinput->get('option') != 'com_rseventspro' || !$this->hasEventFields($args['formId']))
		{
			return;
		}

		require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/rseventspro.php';

		$formId         = $args['formId'];
		$SubmissionId   = $args['SubmissionId'];
		$event          = RSEventsProPaymentHelper::getEvent();

		if ($event && $event->form == $formId)
		{
			$this->result = rseventsproHelper::saveRegistration($SubmissionId);

			if (!$this->result['status'])
			{
				// Remove submission
				require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/submissions.php';
				RSFormProSubmissionsHelper::deleteSubmissions($SubmissionId);

				echo rseventsproHelper::redirect(true,$this->result['message'],$this->result['url'],true);
				JFactory::getApplication()->close();
			}

			$db			= JFactory::getDbo();
			$query		= $db->getQuery(true);
			$from		= $jinput->getInt('from');
			$post		= $jinput->get('form' ,array(), 'array');
			$total		= 0;
			$thestring	= '';
			$ticketsconfig = $event->ticketsconfig;

			if ($ticketsconfig)
			{
				$tickets	= array();
				$thetickets	= $jinput->get('tickets',array(),'array');
				$unlimited	= $jinput->get('unlimited',array(),'array');

				foreach ($thetickets as $tid => $theticket)
				{
					$tickets[$tid] = count($theticket);
				}

				if (!empty($unlimited))
				{
					$unlimited = array_map('intval', $unlimited);
					foreach ($unlimited as $unlimitedid => $quantity)
					{
						$tickets[$unlimitedid] = $quantity;
					}
				}
			}
			else
			{
				if (rseventsproHelper::getConfig('multi_tickets','int'))
				{
					$tickets = $jinput->get('tickets',array(),'array');

					if (empty($tickets) && !empty($post['RSEProTickets']))
					{
						if ($from == 1)
						{
							$tickets = array($post['RSEProTickets'] => $jinput->getInt('number'));
						}
						else
						{
							$tickets = array($post['RSEProTickets'] => $jinput->getInt('numberinp'));
						}
					}
				}
				else
				{
					$ticket = @$post['RSEProTickets'];

					if (!empty($ticket))
					{
						if ($from == 1)
						{
							$tickets = array($ticket => $jinput->getInt('number'));
						}
						else
						{
							$tickets = array($ticket => $jinput->getInt('numberinp'));
						}
					}
				}
			}

			if (!empty($tickets))
			{
				$tmp = array();

				$query->clear()
					->select($db->qn('id'))
					->select($db->qn('name'))
					->select($db->qn('price'))
					->from($db->qn('#__rseventspro_tickets'))
					->where($db->qn('id').' IN (' . implode(',', $db->q(array_keys($tickets))) . ')');

				if ($allTicketDetails = $db->setQuery($query)->loadObjectList('id'))
				{
					foreach ($tickets as $ticket => $quantity)
					{
						if (!isset($allTicketDetails[$ticket]))
						{
							continue;
						}

						$ticketDetails = $allTicketDetails[$ticket];
						$ticketno = $quantity < 0 ? 1 : $quantity;

						if ($ticketDetails->price)
						{
							$tmp[] = $ticketno.' x '.$ticketDetails->name.' ('.rseventsproHelper::currency($ticketDetails->price).')';
							$total += $ticketno * $ticketDetails->price;
						}
						else
						{
							$tmp[] = $ticketno.' x '.$ticketDetails->name.' ('.JText::_('COM_RSEVENTSPRO_GLOBAL_FREE').')';
						}
					}
				}

				$thestring .= !empty($tmp) ? implode(' , ',$tmp) : '';
			}

			$query->clear()
				->select($db->qn('id'))
				->select($db->qn('discount'))
				->select($db->qn('early_fee'))
				->select($db->qn('late_fee'))
				->select($db->qn('tax'))
				->from($db->qn('#__rseventspro_users'))
				->where($db->qn('SubmissionId').' = '.(int) $SubmissionId);

			$db->setQuery($query);
			$paymentInfo = $db->loadObject();

			if (!empty($paymentInfo->discount))
			{
				$total = $total - $paymentInfo->discount;
			}

			if (!empty($paymentInfo->early_fee))
			{
				$total = $total - $paymentInfo->early_fee;
			}

			if (!empty($paymentInfo->late_fee))
			{
				$total = $total + $paymentInfo->late_fee;
			}

			if (!empty($paymentInfo->tax))
			{
				$total = $total + $paymentInfo->tax;
			}

			if ($total)
			{
				$thestring .= ' , '.JText::_('COM_RSEVENTSPRO_GLOBAL_TOTAL').': '.rseventsproHelper::currency($total);
			}

			$payment = @$post['RSEProPayment'];
			$payment = is_array($payment) ? $payment[0] : $payment;

			// Update tickets
			$query->clear()
				->update($db->qn('#__rsform_submission_values'))
				->set($db->qn('FieldValue').' = '.$db->q($thestring))
				->where($db->qn('FieldName').' = '.$db->q('RSEProTickets'))
				->where($db->qn('FormId').' = '.$db->q($formId))
				->where($db->qn('SubmissionId').' = '.$db->q($SubmissionId));

			$db->setQuery($query);
			$db->execute();

			// Update payment method
			$query->clear()
				->update($db->qn('#__rsform_submission_values'))
				->set($db->qn('FieldValue').' = '.$db->q(rseventsproHelper::getPayment($payment)))
				->where($db->qn('FieldName').' = '.$db->q('RSEProPayment'))
				->where($db->qn('FormId').' = '.$db->q($formId))
				->where($db->qn('SubmissionId').' = '.$db->q($SubmissionId));

			$db->setQuery($query);
			$db->execute();
		}
	}

	// After the form has been processed
	public function onRsformFrontendAfterFormProcess($args)
	{
		$jinput	= JFactory::getApplication()->input;

		if (!self::canRun() || $jinput->get('option') != 'com_rseventspro' || !$this->hasEventFields($args['formId']))
		{
			return;
		}

		$db 	= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$event  = RSEventsProPaymentHelper::getEvent();

		if ($event && $event->form == $args['formId'])
		{
			$form = $jinput->get('form',array(),'array');
			if (isset($form['RSEProName']))
			{
				$hasPrice = 0;

				if ($event->ticketsconfig)
				{
					$tickets	= array();
					$thetickets	= $jinput->get('tickets', array(),'array');
					$unlimited	= $jinput->get('unlimited', array(),'array');

					foreach ($thetickets as $tid => $theticket)
					{
						$tickets[$tid] = count($theticket);
					}

					if (!empty($unlimited))
					{
						$unlimited = array_map('intval', $unlimited);
						foreach ($unlimited as $unlimitedid => $quantity)
						{
							$tickets[$unlimitedid] = $quantity;
						}
					}

					if (!empty($tickets))
					{
						$ticket_ids = array_keys($tickets);

						$query->clear()
							->select($db->qn('id'))
							->from($db->qn('#__rseventspro_tickets'))
							->where($db->qn('id').' IN (' . implode(',', $db->q($ticket_ids)) . ')')
							->where($db->qn('price') . ' > 0');

						if ($db->setQuery($query)->loadResult())
						{
							$hasPrice = 1;
						}
					}
				}
				else
				{
					$tickets = $jinput->get('tickets',array(),'array');

					if (empty($tickets))
					{
						$tid = @$form['RSEProTickets'];
						$query->clear()
							->select($db->qn('price'))
							->from($db->qn('#__rseventspro_tickets'))
							->where($db->qn('id').' = '.(int) $tid);

						if ($db->setQuery($query)->loadResult() > 0)
						{
							$hasPrice = 1;
						}
					}
					else
					{
						$query->clear()
							->select($db->qn('id'))
							->from($db->qn('#__rseventspro_tickets'))
							->where($db->qn('id').' IN (' . implode(',', $db->q(array_keys($tickets))) . ')')
							->where($db->qn('price') . ' > 0');

						if ($db->setQuery($query)->loadResult())
						{
							$hasPrice = 1;
						}
					}
				}

				if ($hasPrice && !empty($this->result))
				{
					echo rseventsproHelper::redirect(true,$this->result['message'],$this->result['url'],true);

					JFactory::getApplication()->close();
				}
			}
		}
	}

	// Before the creation of the component body
	public function onRsformBackendBeforeCreateFrontComponentBody($args)
	{
		if (!self::canRun() || JFactory::getApplication()->input->getCmd('option') != 'com_rseventspro' || JFactory::getApplication()->input->getCmd('layout') != 'subscribe')
		{
			return;
		}

		// Allow placeholders to work inside fields
		if (!empty($args['data']['DEFAULTVALUE']))
		{
			$defaulttext = $args['data']['DEFAULTVALUE'];
			$defaulttext = $this->placeholders($defaulttext, JFactory::getApplication()->input->getInt('cid'),'');
			$args['data']['DEFAULTVALUE'] = $defaulttext;
		}

		if (!empty($args['data']['TEXT']))
		{
			$text = $args['data']['TEXT'];
			$text = $this->placeholders($text, JFactory::getApplication()->input->getInt('cid'),'');
			$args['data']['TEXT'] = $text;
		}
	}

	// On form display
	public function onRsformFrontendInitFormDisplay($args)
	{
		if (!self::canRun() || JFactory::getApplication()->input->getCmd('option') == 'com_rseventspro' && JFactory::getApplication()->input->getCmd('layout') == 'subscribe')
		{
			$text = $args['formLayout'];
			$text = $this->placeholders($text, JFactory::getApplication()->input->getInt('cid'), '');
			$args['formLayout'] = $text;
		}
	}

	// After Thankyou message
	public function onRsformFrontendAfterShowThankyouMessage($args)
	{
		$jinput = JFactory::getApplication()->input;

		if (!self::canRun() || $jinput->getCmd('option') != 'com_rseventspro' || $jinput->getCmd('layout') != 'subscribe')
		{
			return;
		}

		$db  	= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$cid 	= $jinput->getInt('cid');

		$query->clear()
			->select($db->qn('name'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$cid);

		$db->setQuery($query);
		$name = $db->loadResult();

		$text = $args['output'];
		$text = $this->placeholders($text,$cid,'');

		if (rseventsproHelper::getConfig('modal') == 0) {
			$replace = '<a class="btn btn-secondary" href="'.rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($cid,$name)).'">'.JText::_('COM_RSEVENTSPRO_GLOBAL_BACK').'</a>';
			$pattern	= '#<input type="button" class="rsform-submit-button btn btn-primary" name="continue"(.*?)/>#is';
			preg_match($pattern,$text,$match);

			if (!empty($match) && isset($match[1])) {
				if (strpos($match[1],'document.location.reload') !== false) {
					$text = preg_replace($pattern,$replace,$text);
				}
			}
		}

		$args['output'] = $text;
	}

	// Before the user email sending procedure
	public function onRsformBeforeUserEmail($args)
	{
		$jinput = JFactory::getApplication()->input;

		if ($jinput->getCmd('option') == 'com_rseventspro' || $jinput->getCmd('layout') == 'subscribe')
		{
			$subjecttext	= $this->placeholders(array('subject' => $args['userEmail']['subject'], 'body' => $args['userEmail']['text']),$jinput->getInt('cid'),'',$args['submissionId']);
			$toreplyto		= $this->placeholders(array('subject' => $args['userEmail']['to'], 'body' => $args['userEmail']['replyto']),$jinput->getInt('cid'),'',$args['submissionId']);
			$ccbcc			= $this->placeholders(array('subject' => $args['userEmail']['cc'], 'body' => $args['userEmail']['bcc']),$jinput->getInt('cid'),'',$args['submissionId']);
			$fromfromName	= $this->placeholders(array('subject' => $args['userEmail']['from'], 'body' => $args['userEmail']['fromName']),$jinput->getInt('cid'),'',$args['submissionId']);

			$args['userEmail']['text'] = $subjecttext['body'];
			$args['userEmail']['subject'] = $subjecttext['subject'];

			$args['userEmail']['to'] = $toreplyto['subject'];
			$args['userEmail']['replyto'] = $toreplyto['body'];

			$args['userEmail']['cc'] = $ccbcc['subject'];
			$args['userEmail']['bcc'] = $ccbcc['body'];

			$args['userEmail']['from'] = $fromfromName['subject'];
			$args['userEmail']['fromName'] = $fromfromName['body'];
		}
	}

	// Before the admin email sending procedure
	public function onRsformBeforeAdminEmail($args)
	{
		$jinput = JFactory::getApplication()->input;

		if ($jinput->getCmd('option') == 'com_rseventspro' || $jinput->getCmd('layout') == 'subscribe')
		{
			$subjecttext	= $this->placeholders(array('subject' => $args['adminEmail']['subject'], 'body' => $args['adminEmail']['text']),$jinput->getInt('cid'),'',$args['submissionId']);
			$toreplyto		= $this->placeholders(array('subject' => $args['adminEmail']['to'], 'body' => $args['adminEmail']['replyto']),$jinput->getInt('cid'),'',$args['submissionId']);
			$ccbcc			= $this->placeholders(array('subject' => $args['adminEmail']['cc'], 'body' => $args['adminEmail']['bcc']),$jinput->getInt('cid'),'',$args['submissionId']);
			$fromfromName	= $this->placeholders(array('subject' => $args['adminEmail']['from'], 'body' => $args['adminEmail']['fromName']),$jinput->getInt('cid'),'',$args['submissionId']);

			$args['adminEmail']['text'] = $subjecttext['body'];
			$args['adminEmail']['subject'] = $subjecttext['subject'];

			$args['adminEmail']['to'] = $toreplyto['subject'];
			$args['adminEmail']['replyto'] = $toreplyto['body'];

			$args['adminEmail']['cc'] = $ccbcc['subject'];
			$args['adminEmail']['bcc'] = $ccbcc['body'];

			$args['adminEmail']['from'] = $fromfromName['subject'];
			$args['adminEmail']['fromName'] = $fromfromName['body'];
		}
	}

	// Before the additional email sending procedure
	public function onRsformBeforeAdditionalEmail($args)
	{
		$jinput = JFactory::getApplication()->input;

		if ($jinput->getCmd('option') == 'com_rseventspro' || $jinput->getCmd('layout') == 'subscribe')
		{
			$subjecttext	= $this->placeholders(array('subject' => $args['additionalEmail']['subject'], 'body' => $args['additionalEmail']['text']),$jinput->getInt('cid'),'',$args['submissionId']);
			$toreplyto		= $this->placeholders(array('subject' => $args['additionalEmail']['to'], 'body' => $args['additionalEmail']['replyto']),$jinput->getInt('cid'),'',$args['submissionId']);
			$ccbcc			= $this->placeholders(array('subject' => $args['additionalEmail']['cc'], 'body' => $args['additionalEmail']['bcc']),$jinput->getInt('cid'),'',$args['submissionId']);
			$fromfromName	= $this->placeholders(array('subject' => $args['additionalEmail']['from'], 'body' => $args['additionalEmail']['fromName']),$jinput->getInt('cid'),'',$args['submissionId']);

			$args['additionalEmail']['text'] = $subjecttext['body'];
			$args['additionalEmail']['subject'] = $subjecttext['subject'];

			$args['additionalEmail']['to'] = $toreplyto['subject'];
			$args['additionalEmail']['replyto'] = $toreplyto['body'];

			$args['additionalEmail']['cc'] = $ccbcc['subject'];
			$args['additionalEmail']['bcc'] = $ccbcc['body'];

			$args['additionalEmail']['from'] = $fromfromName['subject'];
			$args['additionalEmail']['fromName'] = $fromfromName['body'];
		}
	}

	// Get a list of payment methods
	public static function getPayments()
	{
		require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/rseventspro.php';

		return RSEventsProPaymentHelper::getPayments();
	}

	// Get a list of available tickets
	public static function getTickets()
	{
		require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/rseventspro.php';

		return RSEventsProPaymentHelper::getTickets();
	}

	// Add the RSEvents!Pro Emails Tab
	public function onRsformBackendAfterShowFormEditTabsTab()
	{
		?>
		<li><a href="javascript: void(0);" id="rseproemails"><span class="rsficon rsficon-envelope-square"></span><span class="inner-text"><?php echo JText::_('RSFP_RSEPRO_EMAILS'); ?></span></a></li>
		<?php
	}

	// Add the RSEvents!Pro Emails Tab Content
	public function onRsformBackendAfterShowFormEditTabs()
	{
		if (!self::canRun())
		{
			return;
		}

		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$formId	= JFactory::getApplication()->input->getInt('formId',0);
		$table	= JTable::getInstance('RSForm_Rseventspro', 'Table');
		if (!$table)
		{
			return;
		}
		$table->load($formId);

		$form = RSFormProHelper::getForm($formId);
		$lang = $this->getLang($formId);
		if ($lang != $form->Lang)
		{
			$translations = $this->getTranslations($formId, $lang);
			if ($translations)
			{
				foreach ($translations as $field => $value)
				{
					if (property_exists($table, $field))
					{
						$table->{$field} = $value;
					}
				}
			}
		}

		$lists['published']	= RSFormProHelper::renderHTML('select.booleanlist','rsepro[published]','',$table->published);
		$editor				= JEditor::getInstance(JFactory::getConfig()->get('editor'));
		$translateIcon      = RSFormProHelper::translateIcon();
		$emails				= array();

		// Registration email
		$emails['registration'][] = array('label' => JText::_('RSFP_RSEPRO_REGISTRATION'), 'input' => RSFormProHelper::renderHTML('select.booleanlist','rsepro[registration]','',$table->registration));
		$emails['registration'][] = array('label' => $translateIcon.' '.JText::_('RSFP_RSEPRO_REGISTRATION_SUBJECT'), 'input' => '<input type="text" name="rsepro[registration_subject]" value="'.RSFormProHelper::htmlEscape($table->registration_subject).'" class="rs_inp rs_80" size="60" />');
		$emails['registration'][] = array('label' => $translateIcon.' '.JText::_('RSFP_RSEPRO_REGISTRATION_MESSAGE'), 'input' => $editor->display('rsepro[registration_text]', RSFormProHelper::htmlEscape($table->registration_text), '50%', '50%', 90, 15, false, 'registration_text'));

		// Activation email
		$emails['activation'][] = array('label' => JText::_('RSFP_RSEPRO_ACTIVATION'), 'input' => RSFormProHelper::renderHTML('select.booleanlist','rsepro[activation]','',$table->activation));
		$emails['activation'][] = array('label' => $translateIcon.' '.JText::_('RSFP_RSEPRO_ACTIVATION_SUBJECT'), 'input' => '<input type="text" name="rsepro[activation_subject]" value="'.RSFormProHelper::htmlEscape($table->activation_subject).'" class="rs_inp rs_80" size="60" />');
		$emails['activation'][] = array('label' => $translateIcon.' '.JText::_('RSFP_RSEPRO_ACTIVATION_MESSAGE'), 'input' => $editor->display('rsepro[activation_text]', RSFormProHelper::htmlEscape($table->activation_text), '50%', '50%', 90, 15, false, 'activation_text'));

		// Unsubscribe email
		$emails['unsubscribe'][] = array('label' => JText::_('RSFP_RSEPRO_UNSUBSCRIBE'), 'input' => RSFormProHelper::renderHTML('select.booleanlist','rsepro[unsubscribe]','',$table->unsubscribe));
		$emails['unsubscribe'][] = array('label' => $translateIcon.' '.JText::_('RSFP_RSEPRO_UNSUBSCRIBE_SUBJECT'), 'input' => '<input type="text" name="rsepro[unsubscribe_subject]" value="'.RSFormProHelper::htmlEscape($table->unsubscribe_subject).'" class="rs_inp rs_80" size="60" />');
		$emails['unsubscribe'][] = array('label' => $translateIcon.' '.JText::_('RSFP_RSEPRO_UNSUBSCRIBE_MESSAGE'), 'input' => $editor->display('rsepro[unsubscribe_text]', RSFormProHelper::htmlEscape($table->unsubscribe_text), '50%', '50%', 90, 15, false, 'unsubscribe_text'));

		// Denied subscription email
		$emails['denied'][] = array('label' => JText::_('RSFP_RSEPRO_DENIED'), 'input' => RSFormProHelper::renderHTML('select.booleanlist','rsepro[denied]','',$table->denied));
		$emails['denied'][] = array('label' => $translateIcon.' '.JText::_('RSFP_RSEPRO_DENIED_SUBJECT'), 'input' => '<input type="text" name="rsepro[denied_subject]" value="'.RSFormProHelper::htmlEscape($table->denied_subject).'" class="rs_inp rs_80" size="60" />');
		$emails['denied'][] = array('label' => $translateIcon.' '.JText::_('RSFP_RSEPRO_DENIED_MESSAGE'), 'input' => $editor->display('rsepro[denied_text]', RSFormProHelper::htmlEscape($table->denied_text), '50%', '50%', 90, 15, false, 'denied_text'));

		// New event subscription notification email
		$emails['notify'][] = array('label' => JText::_('RSFP_RSEPRO_NOTIFY'), 'input' => RSFormProHelper::renderHTML('select.booleanlist','rsepro[notify]','',$table->notify));
		$emails['notify'][] = array('label' => $translateIcon.' '.JText::_('RSFP_RSEPRO_NOTIFY_SUBJECT'), 'input' => '<input type="text" name="rsepro[notify_subject]" value="'.RSFormProHelper::htmlEscape($table->notify_subject).'" class="rs_inp rs_80" size="60" />');
		$emails['notify'][] = array('label' => $translateIcon.' '.JText::_('RSFP_RSEPRO_NOTIFY_MESSAGE'), 'input' => $editor->display('rsepro[notify_text]', RSFormProHelper::htmlEscape($table->notify_text), '50%', '50%', 90, 15, false, 'notify_text'));

		if (rseventsproHelper::pdf()) {
			$emails['ticketpdf'][] = array('label' => JText::_('RSFP_RSEPRO_TICKET_PDF'), 'input' => RSFormProHelper::renderHTML('select.booleanlist','rsepro[ticketpdf]','',$table->ticketpdf));
			$emails['ticketpdf'][] = array('label' => JText::_('RSFP_RSEPRO_TICKET_PDF_LAYOUT'), 'input' => $editor->display('rsepro[ticketpdf_layout]', RSFormProHelper::htmlEscape($table->ticketpdf_layout), '50%', '50%', 90, 15, false, 'ticketpdf_layout'));
		}

		$tabs = new RSFormProAdapterTabs('com-rsform-rseventspro');

		if (file_exists(JPATH_SITE.'/components/com_rseventspro/helpers/placeholders.php'))
		{
			JFactory::getLanguage()->load('com_rseventspro');
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/placeholders.php';
		}

		JFactory::getDocument()->addStyleDeclaration('#rseproemailsdiv { overflow: hidden; }');
		?>
		<div id="rseproemailsdiv">
			<h3 class="rsfp-legend"><?php echo JText::_('RSFP_RSEPRO_EMAILS'); ?></h3>
			<div class="alert alert-info"><?php echo JText::_('RSFP_RSEPRO_DESC'); ?></div>
			<table class="table table-bordered">
				<tr>
					<td style="width: 1%" nowrap><?php echo JText::_('RSFP_RSEPRO_USE_INTEGRATION'); ?></td>
					<td><?php echo $lists['published']; ?></td>
				</tr>
			</table>
	<?php
	foreach ($emails as $type => $email)
	{
		$tabs->addTitle('RSFP_RSEPRO_'.strtoupper($type).'_EMAIL','rsepro-'.$type);

		if ($type == 'notify')
		{
			$placeholder = 'notify_me';
		}
		elseif ($type == 'ticketpdf')
		{
			$placeholder = 'pdf';
		}
		else
		{
			$placeholder = $type;
		}

		$content = '<table width="100%">';
		$content .= '<tr>';
		$content .= '<td width="60%" valign="top">';

		$content .= '<table class="admintable" width="100%" cellpadding="5">';
		foreach ($email as $field) {
			$content .= '<tr>';
			$content .= '<td width="200px;">';
			$content .= '<label>'.$field['label'].'</label>';
			$content .= '</td>';
			$content .= '<td>';
			$content .= $field['input'];
			$content .= '</td>';
			$content .= '</tr>';
		}
		$content .= '</table>';

		$content .= '</td>';
		$content .= '<td width="5%"></td>';
		$content .= '<td width="35%" valign="top">';

		$content .= '<table class="admintable" width="100%" cellpadding="5">';
		$content .= '<tr>';
		$content .= '<td>';

		try {
			if ($placeholders = RSEventsProPlaceholders::get($placeholder)) {
				$content .= '<table class="admintable" width="100%">';
				$content .= '<thead><tr><th colspan="2">'.JText::_('COM_RSEVENTSPRO_EMAIL_PLACEHOLDERS').'</th></tr></thead>';
				foreach ($placeholders as $name => $description) {
					$content .= '<tr>';
					$content .= '<td>';
					$content .= $name;
					$content .= '</td>';
					$content .= '<td>';
					$content .= JText::_($description);
					$content .= '</td>';
					$content .= '</tr>';
				}
				$content .= '</table>';
			}
		} catch (Exception $e) {}

		$content .= '</td>';
		$content .= '</tr>';
		$content .= '</table>';
		$content .= '</td>';
		$content .= '</tr>';
		$content .= '</table>';

		$tabs->addContent($content);
	}

	$tabs->render();
	?>
		</div>
		<?php
	}

	// Save form
	public function onRsformFormSave($form)
	{
		$app             = JFactory::getApplication();
		$data	         = $app->input->get('rsepro', array(), 'array');
		$data['form_id'] = $form->FormId;
		$table	         = JTable::getInstance('RSForm_Rseventspro', 'Table');

		if (!$table)
		{
			return false;
		}

		$table->bind($data);

		if (!$table->check())
		{
			return false;
		}

		$this->saveTranslation($table, $this->getLang($form->FormId));

		if (!$table->store())
		{
			$app->enqueueMessage($table->getError(), 'error');
		}
	}

	public function onRsformBackendFormCopy($args)
	{
		$formId = $args['formId'];
		$newFormId = $args['newFormId'];

		if ($row = JTable::getInstance('RSForm_Rseventspro', 'Table') )
		{
			if ($row->load($formId))
			{
				if (!$row->bind(array('form_id'=>$newFormId)))
				{
					return false;
				}

				$row->check();
				$row->store();
			}
		}
	}

	// RSEvents!Pro Registration Email
	public function onrseproRegistrationEmail($vars)
	{
		if (!self::canRun())
		{
			return;
		}

		$id	= $vars['ids'];

		if ($table = $this->getTable($id))
		{
			if ($table->registration)
			{
				$subject	= $table->registration_subject;
				$message	= $table->registration_text;

				JFactory::getApplication()->input->set('cid', $table->ide);
				list($replace, $with) = RSFormProHelper::sendSubmissionEmails($table->SubmissionId, true);

				if (strpos($message, '{/if}') !== false) {
					require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/scripting.php';
					RSFormProScripting::compile($message, $replace, $with);
				}

				$subject = str_replace($replace,$with,$subject);
				$message = str_replace($replace,$with,$message);

				$vars['data']['subject']	= $subject;
				$vars['data']['body']		= $message;
			}
		}
	}

	// RSEvents!Pro Activation Email
	public function onrseproActivationEmail($vars)
	{
		if (!self::canRun())
		{
			return;
		}

		$id	= $vars['ids'];

		if ($table = $this->getTable($id))
		{
			if ($table->activation)
			{
				$subject	= $table->activation_subject;
				$message	= $table->activation_text;

				JFactory::getApplication()->input->set('cid', $table->ide);
				list($replace, $with) = RSFormProHelper::sendSubmissionEmails($table->SubmissionId, true);

				if (strpos($message, '{/if}') !== false) {
					require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/scripting.php';
					RSFormProScripting::compile($message, $replace, $with);
				}

				$subject = str_replace($replace,$with,$subject);
				$message = str_replace($replace,$with,$message);

				$vars['data']['subject']	= $subject;
				$vars['data']['body']		= $message;
			}
		}
	}

	// RSEvents!Pro Unsubscribe Email
	public function onrseproUnsubscribeEmail($vars)
	{
		if (!self::canRun())
		{
			return;
		}

		$id	= $vars['ids'];

		if ($table = $this->getTable($id))
		{
			if ($table->unsubscribe)
			{
				$subject	= $table->unsubscribe_subject;
				$message	= $table->unsubscribe_text;

				JFactory::getApplication()->input->set('cid', $table->ide);
				list($replace, $with) = RSFormProHelper::sendSubmissionEmails($table->SubmissionId, true);

				if (strpos($message, '{/if}') !== false) {
					require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/scripting.php';
					RSFormProScripting::compile($message, $replace, $with);
				}

				$subject = str_replace($replace,$with,$subject);
				$message = str_replace($replace,$with,$message);

				$vars['data']['subject']	= $subject;
				$vars['data']['body']		= $message;
			}
		}
	}

	// RSEvents!Pro Denied Email
	public function onrseproDeniedEmail($vars)
	{
		if (!self::canRun())
		{
			return;
		}

		$id	= $vars['ids'];

		if ($table = $this->getTable($id))
		{
			if ($table->denied)
			{
				$subject	= $table->denied_subject;
				$message	= $table->denied_text;

				JFactory::getApplication()->input->set('cid', $table->ide);
				list($replace, $with) = RSFormProHelper::sendSubmissionEmails($table->SubmissionId, true);

				if (strpos($message, '{/if}') !== false) {
					require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/scripting.php';
					RSFormProScripting::compile($message, $replace, $with);
				}

				$subject = str_replace($replace,$with,$subject);
				$message = str_replace($replace,$with,$message);

				$vars['data']['subject']	= $subject;
				$vars['data']['body']		= $message;
			}
		}
	}

	// RSEvents!Pro Notify Email
	public function onrseproNotifyEmail($vars)
	{
		if (!self::canRun())
		{
			return;
		}

		$id	= $vars['ids'];

		if ($table = $this->getTable($id))
		{
			if ($table->notify)
			{
				$subject	= $table->notify_subject;
				$message	= $table->notify_text;

				JFactory::getApplication()->input->set('cid', $table->ide);
				list($replace, $with) = RSFormProHelper::sendSubmissionEmails($table->SubmissionId, true);

				if (strpos($message, '{/if}') !== false) {
					require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/scripting.php';
					RSFormProScripting::compile($message, $replace, $with);
				}

				$subject = str_replace($replace,$with,$subject);
				$message = str_replace($replace,$with,$message);

				$vars['data']['subject']	= $subject;
				$vars['data']['body']		= $message;
			}
		}
	}

	// RSEvents!Pro Ticket PDF
	public function onrseproTicketPDFLayout($vars)
	{
		if (!self::canRun())
		{
			return;
		}

		$id	= $vars['ids'];

		if ($table = $this->getTable($id))
		{
			if ($table->ticketpdf)
			{
				$layout	= $table->ticketpdf_layout;

				JFactory::getApplication()->input->set('cid', $table->ide);
				list($replace, $with) = RSFormProHelper::sendSubmissionEmails($table->SubmissionId, true);

				if (strpos($layout, '{/if}') !== false) {
					require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/scripting.php';
					RSFormProScripting::compile($layout, $replace, $with);
				}

				$vars['layout']	= str_replace($replace,$with,$layout);
			}
		}
	}

	// Check if we can run this plugin
	protected static function canRun()
	{
		if (file_exists(JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/rsform.php') && file_exists(JPATH_SITE.'/components/com_rseventspro/helpers/rseventspro.php'))
		{
			if (!class_exists('RSFormProHelper'))
			{
				require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/rsform.php';
			}

			require_once JPATH_SITE.'/components/com_rseventspro/helpers/rseventspro.php';
			return true;
		}

		return false;
	}
	
	// Replace RSEvents!Pro placeholders
	protected function placeholders($text, $ide, $name, $SubmissionId = null)
	{
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/rseventspro.php';

		if ($ide)
		{
			$optionals	= $SubmissionId ? $this->createOptionals($SubmissionId) : array();
			$ids		= $SubmissionId ? $this->createOptionals($SubmissionId,true) : null;

			return rseventsproEmails::placeholders($text, $ide, $name, $optionals, $ids);
		}

		return $text;
	}
	
	// Create a list of RSEvents!Pro optionals
	protected function createOptionals($id, $ids = false)
	{
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$total	= 0;
		$info	= '';
		
		$optionals = array();
		
		// Get subscription details
		$query->clear()
			->select($db->qn('id'))
			->select($db->qn('ide'))
			->select($db->qn('name'))
			->select($db->qn('email'))
			->select($db->qn('discount'))
			->select($db->qn('early_fee'))
			->select($db->qn('late_fee'))
			->select($db->qn('tax'))
			->select($db->qn('gateway'))
			->select($db->qn('ip'))
			->select($db->qn('coupon'))
			->from($db->qn('#__rseventspro_users'))
			->where($db->qn('SubmissionId').' = '.(int) $id);
		
		$db->setQuery($query);
		$subscription = $db->loadObject();
		
		if ($ids)
		{
			return $subscription->id;
		}
		
		if ($subscription)
		{
			// Get tickets
			$tickets = rseventsproHelper::getUserTickets($subscription->id);
			
			if (!empty($tickets))
			{
				foreach ($tickets as $ticket)
				{
					// Calculate the total
					if ($ticket->price > 0)
					{
						$price = $ticket->price * $ticket->quantity;
						$total += $price;
						$info .= $ticket->quantity . ' x ' .$ticket->name.' ('.rseventsproHelper::currency($ticket->price).') '.rseventsproHelper::getSeats($subscription->id,$ticket->id).' <br />';
					}
					else
					{
						$info .= $ticket->quantity . ' x ' .$ticket->name.' ('.JText::_('COM_RSEVENTSPRO_GLOBAL_FREE').') <br />';
					}
				}
			}
			
			if (!empty($subscription->discount) && !empty($total))
			{
				$total = $total - $subscription->discount;
			}
			
			if (!empty($subscription->early_fee) && !empty($total))
			{
				$total = $total - $subscription->early_fee;
			}
			
			if (!empty($subscription->late_fee) && !empty($total))
			{
				$total = $total + $subscription->late_fee;
			}
			
			if (!empty($subscription->tax) && !empty($total))
			{
				$total = $total + $subscription->tax;
			}
			
			$ticketstotal		= rseventsproHelper::currency($total);
			$ticketsdiscount	= !empty($subscription->discount) ? rseventsproHelper::currency($subscription->discount) : '';
			$subscriptionTax	= !empty($subscription->tax) ? rseventsproHelper::currency($subscription->tax) : '';
			$lateFee			= !empty($subscription->late_fee) ? rseventsproHelper::currency($subscription->late_fee) : '';
			$earlyDiscount		= !empty($subscription->early_fee) ? rseventsproHelper::currency($subscription->early_fee) : '';
			$gateway			= rseventsproHelper::getPayment($subscription->gateway);
			$IP					= $subscription->ip;
			$coupon				= !empty($subscription->coupon) ? $subscription->coupon : '';
			$optionals			= array($info, $ticketstotal, $ticketsdiscount, $subscriptionTax, $lateFee, $earlyDiscount, $gateway, $IP, $coupon);
		}
		
		return $optionals;
	}
	
	// Get details for overriding the RSEvents!Pro emails
	protected function getTable($id)
	{
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		// First, let's assume that we have a SubmissionId
		$query->clear()
			->select($db->qn('rr').'.*')->select($db->qn('u.ide'))
			->select($db->qn('u.lang'))->select($db->qn('u.SubmissionId'))
			->from($db->qn('#__rsform_rseventspro','rr'))
			->join('LEFT',$db->qn('#__rsform_forms','f').' ON '.$db->qn('rr.form_id').' = '.$db->qn('f.FormId'))
			->join('LEFT',$db->qn('#__rsform_submissions','s').' ON '.$db->qn('f.FormId').' = '.$db->qn('s.FormId'))
			->join('LEFT',$db->qn('#__rseventspro_users','u').' ON '.$db->qn('u.SubmissionId').' = '.$db->qn('s.SubmissionId'))
			->where($db->qn('u.id').' = '.(int) $id);
		$db->setQuery($query);
		$table = $db->loadObject();
		
		// If not, then check for a formId in the rseventspro events table
		if (!$table) {
			$query->clear()
				->select($db->qn('rr').'.*')->select($db->qn('u.ide'))
				->select($db->qn('u.lang'))->select($db->qn('u.SubmissionId'))
				->from($db->qn('#__rsform_rseventspro','rr'))
				->join('LEFT',$db->qn('#__rsform_forms','f').' ON '.$db->qn('rr.form_id').' = '.$db->qn('f.FormId'))
				->join('LEFT',$db->qn('#__rseventspro_events','e').' ON '.$db->qn('e.form').' = '.$db->qn('f.FormId'))
				->join('LEFT',$db->qn('#__rseventspro_users','u').' ON '.$db->qn('u.ide').' = '.$db->qn('e.id'))
				->where($db->qn('u.id').' = '.(int) $id);
			$db->setQuery($query);
			$table = $db->loadObject();
		}
		
		if ($table)
		{
			if ($table->published)
			{
				if ($translations = $this->getTranslations($table->form_id, $table->lang))
				{
					foreach ($translations as $field => $value)
					{
						if (property_exists($table, $field))
						{
							$table->{$field} = $value;
						}
					}
				}
				
				return $table;
			}
		}
		
		return false;
	}
	
	// Save RSEvents!Pro Email translations
	protected function saveTranslation(&$table, $lang)
	{
		$form = RSFormProHelper::getForm($table->form_id);

		if ($form->Lang == $lang || (RSFormProHelper::getConfig('global.disable_multilanguage') && RSFormProHelper::getConfig('global.default_language') == 'en-GB'))
		{
			return true;
		}
		
		$fields = array('registration_subject', 'registration_text', 'activation_subject', 'activation_text', 'unsubscribe_subject', 'unsubscribe_text', 'denied_subject', 'denied_text', 'notify_subject', 'notify_text');
		$translations = $this->getTranslations($form->FormId, $lang, 'id');
		$db = JFactory::getDbo();

		foreach ($fields as $field)
		{
			$translation = (object) array(
				'form_id'       => $form->FormId,
				'lang_code'     => $lang,
				'reference'     => 'rseventspro',
				'reference_id'  => $field,
				'value'         => $table->{$field}
			);

			if (!isset($translations[$field]))
			{
				$db->insertObject('#__rsform_translations', $translation);
			}
			else
			{
				$translation->id = $translations[$field];
				$db->updateObject('#__rsform_translations', $translation, array('id'));
			}

			unset($table->{$field});
		}
	}
	
	// Get default form language
	protected function getLang($formId = null)
	{
		return RSFormProHelper::getCurrentLanguage($formId);
	}
	
	// Get translations
	protected function getTranslations($formId, $lang, $select = 'value')
	{
		return RSFormProHelper::getTranslations('rseventspro', $formId, $lang, $select);
	}
	
	public function onRsformFormDelete($formId)
	{
		$db 	= JFactory::getDbo();
		$query 	= $db->getQuery(true);
		$query->delete('#__rsform_rseventspro')
			  ->where($db->qn('form_id').'='.$db->q($formId));
		$db->setQuery($query)->execute();
	}
	
	// Allow this plugin to inject its own settings in the backup.
	public function onRsformFormBackup($form, $xml, $fields)
	{
		$db 	= JFactory::getDbo();
		$query 	= $db->getQuery(true);
		$query->select('*')
			  ->from($db->qn('#__rsform_rseventspro'))
			  ->where($db->qn('form_id').'='.$db->q($form->FormId));
		$db->setQuery($query);
		if ($rseventspro = $db->loadObject())
		{
			// No need for a form_id
			unset($rseventspro->form_id);
			
			$xml->add('rseventspro');
			foreach ($rseventspro as $property => $value)
			{
				$xml->add($property, $value);
			}
			$xml->add('/rseventspro');
		}
	}
	
	public function onRsformFormRestore($form, $xml, $fields)
	{
		if (isset($xml->rseventspro))
		{
			$data = array(
				'form_id' => $form->FormId
			);
			
			foreach ($xml->rseventspro->children() as $property => $value)
			{
				$data[$property] = (string) $value;
			}

			$row = JTable::getInstance('RSForm_Rseventspro', 'Table');
			$row->save($data);
		}
	}
	
	public function onRsformBackendFormRestoreTruncate()
	{
		JFactory::getDbo()->truncateTable('#__rsform_rseventspro');
	}
	
	public function onRsformAfterCreateQuickAddPlaceholders(& $placeholders, $componentId)
	{
		static $done;
		
		if (in_array($componentId, $this->newComponents) && !$done)
		{
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/placeholders.php';

			if ($rseproPlaceholders = RSEventsProPlaceholders::get('payment'))
			{
				$placeholders['display'] = array_merge($placeholders['display'], array_keys($rseproPlaceholders));
			}

			$done = true;
		}

		return $placeholders;
    }

	public function onRsformBackendGetHeaderLabel(&$header, $formId)
	{
		if ($this->hasEventFields($formId))
		{
			if ($header === 'RSEProName')
			{
				$header = JText::_('RSFP_RSEPRO_NAME');
			}
			elseif ($header === 'RSEProEmail')
			{
				$header = JText::_('RSFP_RSEPRO_EMAIL');
			}
			elseif ($header === 'RSEProTickets')
			{
				$header = JText::_('RSFP_RSEPRO_TICKETS');
			}
			elseif ($header === 'RSEProPayment')
			{
				$header = JText::_('RSFP_RSEPRO_PAYMENTS');
			}
			elseif ($header === 'RSEProCoupon')
			{
				$header = JText::_('RSFP_RSEPRO_COUPON');
			}
		}
	}

	protected function hasEventFields($formId)
	{
		static $cache = array();

		if (!isset($cache[$formId]))
		{
			$cache[$formId] = RSFormProHelper::componentExists($formId, $this->newComponents) ? true : false;
		}

		return $cache[$formId];
	}

	public function onRsformBackendCreateConditionOptionFields($args)
	{
		if (!self::canRun()) return;

		$args['types'][] = RSFORM_FIELD_RSEVENTS_PAYMENTS;

		if ($componentId = RSFormProHelper::componentExists($args['formId'], RSFORM_FIELD_RSEVENTS_PAYMENTS))
		{
			$properties =& RSFormProHelper::getComponentProperties($componentId[0]);
			$properties['ITEMS'] = array();

			if ($items = rseventsproHelper::getPayments())
			{
				foreach ($items as $item)
				{
					$properties['ITEMS'][] = $item->value . '|' . $item->text;
				}
			}

			$properties['ITEMS'] = implode("\n", $properties['ITEMS']);
		}
	}
}