<?php
/**
* @package RSForm!Pro
* @copyright (C) 2007-2019 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

// no direct access
defined('_JEXEC') or die('Restricted access');

class plgSystemRsfprseventsproInstallerScript
{
	protected static $minJoomla = '3.7.0';
	protected static $minComponent = '3.1.0';
	
	public function preflight($type, $parent)
	{
		if ($type == 'uninstall')
		{
			return true;
		}
		
		try
		{
			$source = $parent->getParent()->getPath('source');

			$jversion = new JVersion();

			if (!$jversion->isCompatible(static::$minJoomla))
			{
				throw new Exception('Please upgrade to at least Joomla! ' . static::$minJoomla . ' before continuing!');
			}

			if (!file_exists(JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/rsform.php'))
			{
				throw new Exception('Please install the RSForm! Pro component before continuing.');
			}

			if (!file_exists(JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/assets.php') || !file_exists(JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/version.php'))
			{
				throw new Exception('Please upgrade RSForm! Pro to at least version ' . static::$minComponent . ' before continuing!');
			}

			require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/version.php';

			if (!class_exists('RSFormProVersion') || version_compare((string) new RSFormProVersion, static::$minComponent, '<'))
			{
				throw new Exception('Please upgrade RSForm! Pro to at least version ' . static::$minComponent . ' before continuing!');
			}
			
			if (!file_exists(JPATH_ADMINISTRATOR.'/components/com_rseventspro/rseventspro.php') || !file_exists(JPATH_SITE.'/components/com_rseventspro/helpers/version.php'))
			{
				throw new Exception('Please install the RSEvents!Pro component before continuing.');
			}

			require_once JPATH_SITE.'/components/com_rseventspro/helpers/version.php';
			
			$version = new RSEventsProVersion;
			$version = $version->version;
			
			if (!version_compare($version, '1.8.11', '>='))
			{
				throw new Exception('You need to have at least version 1.8.11 of RSEvents!Pro in order to continue.');
			}

			// Update? Run our SQL file
			if ($type == 'update')
			{
				$this->runSQL($source, 'install');
			}

			$this->copyFiles($parent);
		}
		catch (Exception $e)
		{
			JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');

			return false;
		}

		return true;
	}
	
	public function install($parent) {
		$this->copyFiles($parent);
	}
	
	public function update($parent) {
		$db = JFactory::getDbo();
		
		$db->setQuery("SELECT `ComponentTypeId` FROM `#__rsform_component_type_fields` WHERE `FieldName` = 'FLOW' AND `ComponentTypeId` = 33");
		if (!$db->loadResult()) {
			$db->setQuery("INSERT IGNORE INTO `#__rsform_component_type_fields` (`ComponentTypeId`, `FieldName`, `FieldType`, `FieldValues`, `Ordering`) VALUES(33, 'FLOW', 'select', 'HORIZONTAL\r\nVERTICAL', '11')");
			$db->execute();
		}
		
		$this->copyFiles($parent);
		$this->runSQL($parent->getParent()->getPath('source'), 'install');
		
		$table = '#__rsform_rseventspro';
		
		if ($fields = $db->getTableColumns($table, false)) {
			foreach ($fields as $field) {
				$fieldType = strtolower($field->Type);
				$fieldKey = strtolower($field->Key);
				
				if (strpos($fieldType, 'int') !== false || strpos($fieldType, 'float') !== false|| strpos($fieldType, 'decimal') !== false) {
					if ($fieldKey != 'pri') {
						$db->setQuery('ALTER TABLE '.$db->qn($table).' ALTER '.$db->qn($field->Field).' SET DEFAULT '.$db->q(0));
						$db->execute();
					}
				} elseif (strpos($fieldType, 'varchar') !== false) {
					$db->setQuery('ALTER TABLE '.$db->qn($table).' ALTER '.$db->qn($field->Field).' SET DEFAULT '.$db->q(''));
					$db->execute();
				} elseif (strpos($fieldType, 'datetime') !== false) {
					$db->setQuery('ALTER TABLE '.$db->qn($table).' ALTER '.$db->qn($field->Field).' SET DEFAULT '.$db->q($db->getNullDate()));
					$db->execute();
				} elseif (strpos($fieldType, 'text') !== false) {
					$db->setQuery('ALTER TABLE '.$db->qn($table).' CHANGE '.$db->qn($field->Field).' '.$db->qn($field->Field).' TEXT NULL DEFAULT NULL');
					$db->execute();
				}
			}
		}
		
	}

	protected function runSQL($source, $file)
	{
		$db = JFactory::getDbo();
		$sqlfile = $source . '/sql/mysql/' . $file . '.sql';

		if (file_exists($sqlfile))
		{
			$buffer = file_get_contents($sqlfile);
			if ($buffer !== false)
			{
				$queries = $db->splitSql($buffer);
				foreach ($queries as $query)
				{
					$query = trim($query);
					if ($query != '')
					{
						$db->setQuery($query)->execute();
					}
				}
			}
		}
	}
	
	protected function copyFiles($parent) {
		$app = JFactory::getApplication();
		$installer = $parent->getParent();
		$src = $installer->getPath('source').'/admin';
		$dest = JPATH_ADMINISTRATOR.'/components/com_rsform';
		
		if (!JFolder::copy($src, $dest, '', true)) {
			$app->enqueueMessage('Could not copy to '.str_replace(JPATH_ADMINISTRATOR, '', $dest).', please make sure destination is writable!', 'error');
		}
	}
}